package com.example.shawarma.data.api.mediators

import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import com.example.shawarma.data.api.MyServerService
import com.example.shawarma.data.api.models.OrderListResponse
import com.example.shawarma.data.api.models.toOrderModel
import com.example.shawarma.data.db.AppDatabase
import com.example.shawarma.data.db.models.OrderModel
import com.example.shawarma.data.db.models.OrderWithProducts
import java.io.IOException

@OptIn(ExperimentalPagingApi::class)
class OrderRemoteMediator (
    private val database: AppDatabase,
    private val serverService: MyServerService,
    private val query: String,
    private val token: String
) : RemoteMediator<Int, OrderWithProducts>(){

    private val orderDao = database.orderDao()

    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, OrderWithProducts>
    ): MediatorResult {
        return try {
            var loadKey = when (loadType) {
                LoadType.REFRESH -> null
                LoadType.PREPEND -> return MediatorResult.Success(endOfPaginationReached = true)
                LoadType.APPEND -> {
                    val lastItem = state.lastItemOrNull()
                        ?: return MediatorResult.Success(endOfPaginationReached = true)
                    lastItem.order.id
                }
            }

            if (loadKey == null) {
                loadKey = 0
            }
            val response: OrderListResponse = when (query) {
                "paid" -> {
                    serverService.getPaidOrdersList(after = loadKey, token = token)
                }
                "all_paid" -> {
                    serverService.getAllPaidOrdersList(after = loadKey, token = token)
                }
                "all_ready" -> {
                    serverService.getAllReadyOrdersList(after = loadKey, token = token)
                }
                "all_served" -> {
                    serverService.getAllServedOrdersList(after = loadKey, token = token)
                }

                else -> {
                    OrderListResponse()
                }
            }


            database.withTransaction {
                if (loadType == LoadType.REFRESH) {
                    orderDao.deleteByQuery(query)
                }

                val orders = mutableListOf<OrderModel>()
                for (order in response.orders) {
                    orders.add(order.toOrderModel())
                }
                orderDao.insertAll(orders)
            }

            MediatorResult.Success(endOfPaginationReached = response.nextKey == -1)
        } catch (e: IOException) {
            MediatorResult.Error(e)
        }
    }
}