package com.example.shawarma.data.api.mediators

import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import com.example.shawarma.data.api.MyServerService
import com.example.shawarma.data.api.models.ProductListResponse
import com.example.shawarma.data.api.models.toProductModel
import com.example.shawarma.data.db.AppDatabase
import com.example.shawarma.data.db.models.ProductModel
import kotlinx.coroutines.flow.first
import java.io.IOException

@OptIn(ExperimentalPagingApi::class)
class ProductRemoteMediator (
    private val database: AppDatabase,
    private val serverService: MyServerService,
    private val query: String,
    private val token: String
) : RemoteMediator<Int, ProductModel>(){

    private val productDao = database.productDao()

    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, ProductModel>
    ): MediatorResult {
        return try {
            var loadKey = when (loadType) {
                LoadType.REFRESH -> null
                LoadType.PREPEND -> return MediatorResult.Success(endOfPaginationReached = true)
                LoadType.APPEND -> {
                    val lastItem = state.lastItemOrNull()
                        ?: return MediatorResult.Success(endOfPaginationReached = true)
                    lastItem.id
                }
            }

            if (loadKey == null) {
                loadKey = 0
            }
            val response: ProductListResponse = when (query) {
                "home" -> {
                    serverService.getProductsList(after = loadKey, token = token)
                }
                "discounts" -> {
                    serverService.getDiscountsList(after = loadKey, token = token)
                }
                "items" -> {
                    serverService.getItemsList(after = loadKey, token = token)
                }
                else -> {ProductListResponse()}
            }

            database.withTransaction {

                if (loadType == LoadType.REFRESH) {
                    if (query != "items") {
                        productDao.deleteByQuery(query)
                    }
                    else {
                        val present = productDao.getItemsByLoadKey(loadKey).first()
                        val responseProducts = response.products.map { it -> it.toProductModel() }
                        for (product in present) {
                            if (product !in responseProducts ) {
                                productDao.delete(product)
                            }
                        }
                    }
                }

                val products = mutableListOf<ProductModel>()
                for (prod in response.products) {
                    products.add(prod.toProductModel())
                }
                productDao.insertAll(products)
            }

            MediatorResult.Success(endOfPaginationReached = response.nextKey == -1)
        } catch (e: IOException) {
            MediatorResult.Error(e)
        }
    }
}