package com.example.shawarma.data.repos

import androidx.paging.ExperimentalPagingApi
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import com.example.shawarma.data.api.MyServerService
import com.example.shawarma.data.api.mediators.OrderRemoteMediator
import com.example.shawarma.data.api.models.toOrderModel
import com.example.shawarma.data.api.models.toOrderModelRemote
import com.example.shawarma.data.api.models.toOrderProductModel
import com.example.shawarma.data.api.repos.RestOrderRepository
import com.example.shawarma.data.db.AppDatabase
import com.example.shawarma.data.db.dao.OrderDao
import com.example.shawarma.data.db.dao.OrderProductDao
import com.example.shawarma.data.db.models.OrderModel
import com.example.shawarma.data.db.models.OrderWithProducts
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first
import javax.inject.Inject

class OrderRepository @Inject constructor(
    private val database: AppDatabase,
    private val orderDao: OrderDao,
    private val orderProductDao: OrderProductDao,
    private val restRepository: RestOrderRepository
){
    suspend fun insert(token: String, order: OrderModel): OrderModel? {
        orderDao.insert(order)
        val result =  restRepository.insertOrder(token, order.toOrderModelRemote())
        if (result == null) {
            return result
        }
        return result.toOrderModel()
    }
    suspend fun update(token: String, order: OrderModel) {
        orderDao.update(order)
        restRepository.updateOrder(token, order.toOrderModelRemote())
    }
    suspend fun delete(token: String, order: OrderModel) {
        orderProductDao.deleteByOrderId(order.id!!)
        orderDao.delete(order)
        restRepository.deleteOrder(token, order.id!!)
    }
    fun getById(id: Int): Flow<OrderWithProducts?>{
        return orderDao.getById(id)
    }
    suspend fun getUnpaidByUser(token: String) : OrderWithProducts? {
        val order = restRepository.getUnpaidOrder(token)
        if (order!!.id == null) {
            return null
        }
        if (order!!.id != null) {
            orderDao.insert(order.toOrderModel())
            for (product in order.order_products) {
                orderProductDao.insert(product.toOrderProductModel())
            }
            return orderDao.getUnpaidByUser(order.user_id!!).first()
        }
        return null
    }

    @OptIn(ExperimentalPagingApi::class)
    fun getPaidPaged(token: String): Flow<PagingData<OrderWithProducts>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = orderDao::getPaidPaged,
        remoteMediator = OrderRemoteMediator(database = database, serverService = MyServerService.getInstance(), query = "paid", token = token)
    ).flow

    @OptIn(ExperimentalPagingApi::class)
    fun getAllPaidPaged(token: String): Flow<PagingData<OrderWithProducts>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = orderDao::getAllPaid,
        remoteMediator = OrderRemoteMediator(database = database, serverService = MyServerService.getInstance(), query = "all_paid", token = token)
    ).flow

    @OptIn(ExperimentalPagingApi::class)
    fun getAllReadyPaged(token: String): Flow<PagingData<OrderWithProducts>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = orderDao::getAllReady,
        remoteMediator = OrderRemoteMediator(database = database, serverService = MyServerService.getInstance(), query = "all_ready", token = token)
    ).flow

    @OptIn(ExperimentalPagingApi::class)
    fun getAllServedPaged(token: String): Flow<PagingData<OrderWithProducts>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = orderDao::getAllServed,
        remoteMediator = OrderRemoteMediator(database = database, serverService = MyServerService.getInstance(), query = "all_served", token = token)
    ).flow


}